import * as http from 'http';
import { SioRpcServer, SioRpcClient } from './main';

(async function main() {
  try {
    const server = http.createServer();
    const port = 42080;
    server.listen(port, async () => {
      try {
        const rpcServer = new SioRpcServer(server);
        rpcServer.declare('com.sqreept.siorpc.remoteAdd', (firstTerm: number, secondTerm: number) => firstTerm + secondTerm);
        rpcServer.declare('com.sqreept.siorpc.remoteAddAsync', (firstTerm: number, secondTerm: number) => {
          return new Promise((resolve, reject) => {
            setTimeout(() => reject(new RangeError('Random error')), 1000);
          });
        });
        rpcServer.run();
        const rpcClient = new SioRpcClient(`http://localhost:${port}`);
        const value1 = await rpcClient.call('com.sqreept.siorpc.remoteAdd', 34, 6);
        console.log('value1: ', value1);
        const value2 = await rpcClient.call('com.sqreept.siorpc.remoteAddAsync', 34, 8);
        console.log('value2: ', value2);
        rpcClient.subscribe('com.sqreept.siorpc.onLowMemory', (freeRam: number) => {
          console.log('freeRam: ', freeRam);
        });
        rpcServer.publish('com.sqreept.siorpc.onLowMemory', 3096);
      } catch (e) {
        console.error(e.stack);
        process.exit(1);
      }
    });
  } catch (e) {
    console.error(e.stack);
    process.exit(1);
  }
}());
